class ShiftsController < ApplicationController
  model_data_store(Shift)

  def create
    Shift.transaction do
      @model = model_type.new(__mssql_model_params__.except(:id, :shift_schedules))
      @model.shift_schedules = @model.shift_schedules.build((__mssql_model_params__[:shift_schedules] || []))
      if @model.save
        render json: @model, status: :created, location: @model
      else
        render json: @model.errors, status: :unprocessable_entity
        raise ActiveRecord::Rollback
      end
    end
  end

  def update
    Shift.transaction do
      @model.shift_schedules.each(&:destroy)
      @model.shift_schedules = @model.shift_schedules.build((__mssql_model_params__[:shift_schedules] || []))
      if @model.update(__mssql_model_params__.except(:id, :shift_schedules))
        render json: @model, status: :ok, location: @model
      else
        render json: @model.errors, status: :unprocessable_entity
        raise ActiveRecord::Rollback
      end
    end
  end

  def instances
    @start_date = instance_params[:start].to_time
    @end_date = instance_params[:end].to_time
    @models = Shift.instances_for(@start_date, @end_date, instance_params)
  end

  def active_ranges_between
    @start_date = active_range_params[:start].to_time
    @end_date = active_range_params[:end].to_time
    @include_details = active_range_params[:include_details]
    @ranges = Shift.active_ranges_between(@start_date, @end_date)
    @active_seconds = @ranges.reduce(0) do |sum, range|
      sum + (range.end - range.begin).to_i
    end
  end

  def current_shifts
    @omit_rules = true
    @time = Time.now
    @models = Shift.instances_for(@time, @time)
  end

  def preview
    @start_date = preview_params[:start].to_time
    @end_date = preview_params[:end].to_time
    @dates = preview_params[:shift_schedules].each_with_object([]) do |schedule, bucket|
      this_start_time = schedule[:start_date]&.to_time
      this_end_time = schedule[:end_date]&.to_time
      capped_end = if this_end_time && (this_end_time <= @end_date)
        this_end_time
      elsif this_end_time
        this_end_time.change(year: @end_date.year, month: @end_date.month, day: @end_date.day)
      else
        @end_date
      end
      bucket.concat(
        # this has the potential to be infinite/lockup, so we timeout if the rule has a validity error
        ShiftSchedule.occurrences_between(this_start_time, capped_end, schedule[:shift_length], schedule[:recurrence_rule]).map do |date|
          {
            start: date.start_time,
            end: date.end_time
          }
        end
      )
    end
  end

  private

  def applicable_authorization
    # we suppress the read check so other modules can call this.  We maintain the read permission for frontend purposes though.
    auth = super
    if auth && auth[:id].to_s == "read"
      false
    else
      auth
    end
  end

  # Only allow a list of trusted parameters through.
  def model_params
    unless @trusted_params
      @trusted_params = params.require(:shift).permit(:name, :color, :shift_schedules)
      @trusted_params[:shift_schedules] = JSON.parse(@trusted_params[:shift_schedules]) if @trusted_params[:shift_schedules]&.is_a?(String)
      @trusted_params[:shift_schedules] = @trusted_params[:shift_schedules]&.compact&.map do |schedule|
        trusted_schedule = schedule.permit!.to_h
        trusted_schedule = trusted_schedule.slice(:start_date, :end_date, :shift_length, :recurrence_rule)
        trusted_schedule[:start_date] = trusted_schedule[:start_date].to_time.change(sec: 0) if trusted_schedule[:start_date]
        trusted_schedule[:end_date] = trusted_schedule[:end_date].to_time.change(sec: 0) if trusted_schedule[:end_date]
        trusted_schedule
      end
      @trusted_params[:shift_schedules]&.each do |schedule|
        schedule.permit!
      end
    end
    @trusted_params
  end

  def active_range_params
    unless @trusted_params
      params.require(:start)
      params.require(:end)
      @trusted_params = params.permit(:start, :end, :include_details)
      @trusted_params[:include_details] = @trusted_params[:include_details]&.downcase == "true"
    end
    @trusted_params
  end

  def instance_params
    unless @trusted_params
      params.require(:start)
      params.require(:end)
      @trusted_params = params.permit(:start, :end, :shift_id)
    end
    @trusted_params
  end

  def preview_params
    unless @trusted_params
      params.require(:start)
      params.require(:end)
      @trusted_params = params.permit(:start, :end, shift_schedules: [:start_date, :end_date, :shift_length, recurrence_rule: []])
      @trusted_params[:shift_schedules]&.each_with_index do |schedule, i|
        schedule[:recurrence_rule] = params[:shift_schedules][i].permit![:recurrence_rule]
      end
    end
    @trusted_params
  end
end
