import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import {LoggerConfigTransport} from "../transports/LoggerConfigTransport";
import * as yup from 'yup';

const filePathRegex = /(?:^[a-zA-Z]:(?:\\|\/).*|\\\\.+\\.+$)|(?:^$)/i; // (loosely) validate windows file path format (or empty)

@model({
    transportLayer: new LoggerConfigTransport()
})
export class LoggerConfigModel extends Model {
    @attribute({
        label: t`Location`,
        validation: yup.string().nullable().matches(filePathRegex, t`PDF Directory must be a properly formatted Windows file path if provided`)
    })
    location = "";

    @attribute({label: t`Level`})
    level = "";

    @attribute({label: t`Archive Count`})
    archive_count = 0;

    @attribute({label: t`Max Size (Bytes)`})
    max_size = 0;

    @attribute({autoForm: false})
    module_id = "";

    constructor() {
        super();
        super.setup();
    }
}

export const LoggerConfigStore = LoggerConfigModel.store as unknown as DataStore<LoggerConfigModel>;